unit usqldbruntime;

{$mode ObjFPC}{$H+}

{Switches:
You can disable switches by using a period after the curved
bracket like this  {.$DEFINE SWITCHNAME}
}

{delete Databasefile before ZConnection1.Open so on every application
 start a new and empty file is created}
{.$DEFINE DeleteDatabasefileBeforeOpen}

{Import Data automatically on Create. You can use the procedure ImportData to import
data at any given time}
{.$DEFINE ImportDemoDataOnCreate}

{Note: For a demo or sample application it can be advantageous
 that the same initial data is available at every program start.
 In this case activate both switches}

//*********extended settings:

{By default, based on the selected connection type, the default library
for that connection type is used. If you want to use your own library,
activate this switch. The name of the own library has to be entered under
cSQLdbLibraryName in the include file. Note the differences in the name
under Windows and Linux.}
{.$DEFINE UseCustomLibrary}

{If you want to use a SQL log  enable the switch}
{.$DEFINE UseSQLLOG}

{If you want to use an additional possibility to query
 the database beside other Queries in this class then enable the switch - SQLQuery1 will be created}
{.$DEFINE UseAdditionalQuery}

interface

uses
  Classes, SysUtils, SQLDBLib, SQLDB, Forms, uguessfile, dialogs
  {$ifdef UseSQLLOG}
  , LazLogger
  {$endif}
  , SQLite3Conn

  {if you want to use other database connections than SQLite you need
   the corresponding includes activated}

  //,pqconnection     //PostgreSQ
  //,ibconnection     //Interbase ,Firebird
  //,mssqlconn        //MSSQL Server , Sybase
  //,oracleconnection //Oracle
  //,odbcconn         //ODBC Access
  //,mysql40conn      //different MySQL Versions as well as MariaDB
  //,mysql41conn
  //,mysql50conn
  //,mysql51conn
  //,mysql55conn
  //,mysql56conn
  //,mysql57conn
  //,mysql80conn
  ;

const
  {$include demosqlite.Inc}
  cSQLiteType='SQLite3';

//TSQLConnType, TSQLServerType  copied from SQLDBToolsunit.pas therefore they may be unsynchronized
type
  TSQLServerType = (ssFirebird, ssInterbase, ssMSSQL, ssMySQL, ssOracle, ssPostgreSQL, ssSQLite, ssSybase, ssUnknown);


type

  { Tsqldbruntime }

  Tsqldbruntime = class(TObject)
    SQLDBLibraryLoader1: TSQLDBLibraryLoader;
    SQLTransaction1: TSQLTransaction;
    {$ifdef UseSQLLOG}
    procedure DoSQLLog(Sender: TSQLConnection; EventType: TDBEventType; const Msg: String);
    {$endif}
  private
    FsConnectorType: string;
    FsPossibleConnectorTypes: string;
    //SQLConnType : TSQLConnType;
    SQLServerType : TSQLServerType;
    procedure SetConnectorType(AValue: string);
  public
    SQLConnector1: TSQLConnector;
    {$ifdef UseAdditionalQuery}
    SQLQuery1: TSQLQuery;
    {$endif}
    constructor Create;
    destructor Destroy;
    procedure ConnectToDatabase;
    procedure ImportData(sImportFile:string='');
    procedure OpenConnection;
    procedure CloseConnection;
    {lists the available connection types depending on the SQL units included in the uses section}
    function ListConnectionTypes(bCreateText:boolean=true):string;

    {determines the standard library for a given connection type depending on the SQL units included in the uses section }
    function GetStandardLibraryName(sConnector:string):string;

    {determines and loads the SQL access library depending on the different operating systems and the selected connection type}
    procedure MyLoadLib(sConnectionType,sLibname : String);
    function MyGetLibraryName:string;

    {returns the current SQLite Version if SQLite is chosen as ConnectorType}
    function GetSQLiteVersion:string;

    {In case you need an additional TSQLQuery}
    function GetQuery : TSQLQuery;

    property psPossibleConnectorTypes:string read FsPossibleConnectorTypes;
    property psConnectorType:string read FsConnectorType write SetConnectorType;
  end;


implementation

{ Tsqldbruntime }


procedure Tsqldbruntime.ConnectToDatabase;
var
  sLibrary:string;
  sSQLfile:string;
begin
  CloseConnection;                                    //in case SQLDBLibraryLoader1 or SQLConnector1 were active

  //First
  sLibrary := MyGetLibraryName;
  if sLibrary = cUNDEF then exit;                     //for security reasons in case the exception fails

  //Second: load library
  MyLoadLib(cConnectorType,sLibrary);

  //Third:
  // Try to find the Database file for file based systems or set credentials for others.
  // If database file for file bases systems do not exist create the named database file automatically

  //distinguish file databases versus server based databases
  case SQLServerType of
    ssSQLite,
    ssInterbase,
    ssFirebird  : begin
      {$ifdef DeleteDatabasefileBeforeOpen}
       sSQLFile := GuessSQLDatabaseFile(true);
      {$else}
       sSQLFile := GuessSQLDatabaseFile(false);
      {$endif}
      SQLConnector1.DatabaseName:=sSQLFile;
    end
    else begin
      //feel free to get credentials elsewhere - from an INI file for example
      SQLConnector1.DatabaseName:=cSQLDatabasename;
      SQLConnector1.UserName:=cSQLuser;
      SQLConnector1.Password:=cSQLpassword;
      SQLConnector1.Hostname:=cSQLhostname;
    end;
  end;

  SQLConnector1.ConnectorType:=FsConnectorType;
  SQLConnector1.CharSet:='UTF8';
  SQLConnector1.LoginPrompt := False;
  SQLConnector1.KeepConnection := False;
  SQLConnector1.Options:=[scoApplyUpdatesChecksRowsAffected] ;

  //+[sqImplicitTransaction];
  {
    TConnOption = (sqSupportParams, sqSupportEmptyDatabaseName, sqEscapeSlash, sqEscapeRepeat, sqImplicitTransaction, sqLastInsertID, sqSupportReturning,sqSequences);
  TConnOptions= set of TConnOption;


  Both sqoKeepOpenOnCommit and CommitRetaining will keep the result set open but they are not the same for transaction handling.

  Using sqoKeepOpenOnCommit with Commit will keep the result set and close the transaction.

  Using CommitRetaining will keep the result set and open a new transaction. I do not recommend using CommitRetaining unless you have a good reason for keeping an open transaction.

 ******************
 uses Classes, Sysutils, SQLDB (* und weitere Units: z.B. SQLite3conn *);
 var
   ListOfConnectors: TStrings;
   ConnectorTypeName: String;
   ConnectorDefinition: TConnectionDef;
 begin
   ListOfConnectors := TStringList.Create;
   for ConnectorTypeName in ListOfConnectors do
   begin
     ConnectorDefinition := GetConnectionDef(ConnectorTypeName);
     WriteLn(ConnectorDefinition.TypeName);
     WriteLn(ConnectorDefinition.Description);
     WriteLn(ConnectorDefinition.DefaultLibraryName);
   end;
   ListOfConnectors.Destroy;
 end.
 ******************
  }

  {$ifdef UseSQLLOG}
  SQLConnector1.OnLog:=@DoSQLLog;
  SQLConnector1.LogEvents:=LogAllEventsExtra;  //either LogAllEvents or LogAllEventsExtra
  {$endif}

  OpenConnection;

end;

procedure Tsqldbruntime.SetConnectorType(AValue: string);
begin
  if FsConnectorType=AValue then Exit;
  FsConnectorType:=AValue;
end;

constructor Tsqldbruntime.Create;
begin
  FsPossibleConnectorTypes:=ListConnectionTypes(false);
  FsConnectorType:= cConnectorType;

  SQLConnector1:= TSQLConnector.Create(nil);
  SQLDBLibraryLoader1:=TSQLDBLibraryLoader.Create(nil);

  SQLTransaction1 := TSQLTransaction.Create(nil);
  //tweak Transaction behaviour
  //SQLTransaction1.Options:=[stoUseImplicit];
  //stoExplicitStart
  SQLTransaction1.DataBase:=SQLConnector1;
  //SQLTransaction1.Params.Add('version1');
  //SQLTransaction1.Params.Add('read_committed');
  //SQLTransaction1.Params.Add('isc_tpb_lock_read=PARMTAB');


  {
  procedure TSQLTransaction.SetOptions(AValue: TSQLTransactionOptions);
  begin
    if FOptions=AValue then Exit;
    if (stoUseImplicit in Avalue) and Assigned(SQLConnection) And Not (sqImplicitTransaction in SQLConnection.ConnOptions) then
      DatabaseErrorFmt(SErrNoImplicitTransaction, [SQLConnection.ClassName]);
    FOptions:=AValue;
  end;
  }


  //SQLConnector1.Transaction:= SQLTransaction1;

  {$ifdef UseAdditionalQuery}
  SQLQuery1:=TSQLQuery.Create(nil);
  SQLQuery1.Database:=SQLdbConnection;
  {$endif}

  ConnectToDatabase;

  //Import SQL File (demo data, database structure etc)
  {$ifdef ImportDemoDataOnCreate}
    ImportData;
  {$endif}
end;

destructor Tsqldbruntime.Destroy;
begin
   CloseConnection;
   if SQLDBLibraryLoader1.Enabled then SQLDBLibraryLoader1.Enabled:=false;

   {$ifdef UseAdditionalQuery}
   if SQLQuery1.Active then SQLQuery1.Close;
   SQLQuery1.free;
   {$endif}

   SQLDBLibraryLoader1.free;
   SQLConnector1.free;
   SQLTransaction1.free;

end;

procedure Tsqldbruntime.ImportData(sImportFile: string);
var
  MyScript:TSQLScript;
begin

  if sImportfile = '' then begin
     sImportFile := GuessSQLImportFile(true);
     if sImportFile = cUNDEF then begin //just in case
        showmessage( 'SQL import file not found');
     end;
  end;
  if sImportFile=cUNDEF then  exit;

  if not FileExists(sImportFile) then begin
     showmessage('ImportData: File ' + sImportFile +'does not exist');
     exit;
  end;

  MyScript:=TSQLScript.Create(nil);
  try
    MyScript.Transaction:=SQLTransaction1;
    MyScript.Terminator:=';';
    MyScript.DataBase:=SQLConnector1;
    MyScript.CommentsInSQL:=true;
    MyScript.UseCommit:=true;
    MyScript.UseDefines:=true;
    MyScript.UseSetTerm:=true;
    MyScript.Script.LoadFromFile(sImportFile);
    MyScript.Execute;
    SQLTransaction1.Commit;
  finally
    freeandnil(MyScript);
  end;

end;

procedure Tsqldbruntime.OpenConnection;
begin
  if not SQLConnector1.Connected then
  try
    SQLConnector1.Open;
  except
    On E : Exception do
      begin
        showmessage( LineEnding
                    + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                    + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                    + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                    + 'Connection to Database failed '  + LineEnding+ LineEnding
                    + E.Message);
        Exit;
      end;
  end;
end;

procedure Tsqldbruntime.CloseConnection;
begin
  if SQLConnector1.Connected then SQLConnector1.Close;
end;

function Tsqldbruntime.ListConnectionTypes(bCreateText: boolean): string;
Var
  S : TStringList;
  I : Integer;

begin
  S:=TStringList.Create;
  result:='';
  try
    getConnectionList(S);
    if s.Count=0 then showmessage('keine connections');

    if bCreateText then begin

      result:=LineEnding +'Available connection types:';
      For I:=0 to S.Count-1 do begin
        result:= result
                 + LineEnding
                 + S[i] + LineEnding
                 + 'Default library name: ' + GetConnectionDef(S[i]).DefaultLibraryName + LineEnding
                 + 'ConnectorType: ' + GetConnectionDef(S[i]).TypeName
                 ;
      end;
    end
    else
    begin
      For I:=0 to S.Count-1 do begin
        if i>0 then result:=result+';';
        result:=result+GetConnectionDef(S[i]).TypeName;
      end;
    end;

  finally
    S.free;
  end;

end;


function Tsqldbruntime.GetStandardLibraryName(sConnector: string): string;
var
  S : TStringList;
  I : Integer;
begin
  result:=cUNDEF;
  S:=TStringList.Create;
  try
    getConnectionList(S);
    For I:=0 to S.Count-1 do
      if S[i] = sConnector then begin
        result := GetConnectionDef(S[i]).DefaultLibraryName;
        exit;
      end;
  finally
    S.free;
  end;
end;

procedure Tsqldbruntime.MyLoadLib(sConnectionType, sLibname: String);
begin
  //Using Loadlibrary and unloadlibrary instead of enabling switch to catch exceptions
  try
    SQLDBLibraryLoader1.ConnectionType:=sConnectionType;
    SQLDBLibraryLoader1.UnLoadLibrary;
  except
    On E : Exception do
      begin
        showmessage( LineEnding
                          + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                          + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                          + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                          + 'Error unloading library : '+ SQLDBLibraryLoader1.LibraryName
                          + LineEnding + LineEnding
                          + E.Message);

        Exit;
      end;
  end;

  SQLDBLibraryLoader1.ConnectionType:=sConnectionType; //define connector Type
  SQLDBLibraryLoader1.LibraryName:=sLibname;
  try
    SQLDBLibraryLoader1.LoadLibrary;
  except
    On E : Exception do
      begin
        showmessage( LineEnding
                          + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                          + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                          + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                          + 'Error loading library : '+ sLibName
                          + LineEnding + LineEnding
                          + E.Message);

        Exit;
      end;
  end;
end;

function Tsqldbruntime.MyGetLibraryName: string;
{$ifndef UseCustomLibrary}
var
  sStdLibName:string;
{$endif}
begin

  {$ifndef UseCustomLibrary}
  sStdLibName:='';
  sStdLibName:= GetStandardLibraryName(cConnectorType);
  {$endif}

  result:=cUNDEF;
  {$IFDEF UNIX}
    {$ifdef RasPi}
      result = '????';
    {$else}
      {$ifdef DARWIN}

        {$ifdef UseCustomLibrary}
          result='/usr/lib/'+cSQLdbLibraryName;
        {$else}
          result='/usr/lib/'+sStdLibName;
        {$endif}

      {$else}

        {$ifdef UseCustomLibrary}
          result :=cSQLdbLibraryName;
        {$else}
          result := sStdLibName;
        {$endif}

      {$endif}
    {$endif}
  {$ELSE}       //windows OS
    {$ifdef UseCustomLibrary}
      result := GuessLibraryLocation(cSQLdbLibraryName);
    {$else}
      result :='';
      result := GuessLibraryLocation(sStdLibName);                   //Search for sqlite Library
    {$endif}
  {$ENDIF}
end;

function Tsqldbruntime.GetSQLiteVersion: string;
var
  SQLQ:TSQLQuery;
begin
  result:='No SQLite3 Connection';

  if SQLConnector1.ConnectorType=cSQLiteType then begin
    SQLQ:=TSQLQuery.Create(nil);
    try
      SQLQ.DataBase := SQLConnector1;
      SQLQ.SQL.Text:='SELECT sqlite_version() AS version';
      SQLQ.Open;
      SQLQ.First;
      result:=SQLQ.FieldByName('version').AsString;
      SQLQ.Close;
    finally
      freeandnil(SQLQ);
    end;
  end;

end;

{In case you need an additional TSQLQuery use it like
procedure CreateTable;
var Query : TSQLQuery;
begin
  Query := GetQuery;
  Query.SQL.Text := 'create table TBLNAMES (ID integer, NAME varchar(40));';
  Query.ExecSQL;
  Query.Close;
  Query.Free;
end;}
function Tsqldbruntime.GetQuery: TSQLQuery;
var
  AQuery : TSQLQuery;
begin
  AQuery := TSQLQuery.Create(nil);
  AQuery.Database := SQLConnector1;
  AQuery.Transaction := SQLTransaction1;
  Result := AQuery;
end;


{$ifdef UseSQLLOG}
procedure Tsqldbruntime.DoSQLLog(Sender: TSQLConnection;
  EventType: TDBEventType; const Msg: String);
begin
  //just a simple message to LazLogger
  //Feel free to enhance it
  DebugLn('SQLdb Log-> '+ Msg);
end;
{$endif}

end.

